/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2013, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 * 
 *                       MSP430 CODE EXAMPLE DISCLAIMER
 *
 * MSP430 code examples are self-contained low-level programs that typically
 * demonstrate a single peripheral function or device feature in a highly
 * concise manner. For this the code may rely on the device's power-on default
 * register values and settings such as the clock configuration and care must
 * be taken when combining code from several examples to avoid potential side
 * effects. Also see www.ti.com/grace for a GUI- and www.ti.com/msp430ware
 * for an API functional library-approach to peripheral configuration.
 *
 * --/COPYRIGHT--*/
//******************************************************************************
//  MSP430G6021 Demo - Configure DCO to 1MHz during entry LPM3.
//
//  Description: Before entering LPM3 mode, decrease MCLK frequency to 1MHz in 
//               sequential steps:[8MHz, 16MHz] -> [2MHz, 4MHz] -> 1MHz
//               After exiting LPM3 mode, increase MCLK frequency in sequential
//               steps: 1MHz ->[2MHz, 4MHz] -> [8MHz, 16MHz]
// 
//           MSP430G6021
//         ---------------
//     /|\|               |
//      | |               |
//      --|RST            |
//        |               |

//
//
//  Wei Zhao
//  Texas Instruments Inc.
//  Jan 2014
//  Built with IAR Embedded Workbench Version: v5.60 & Code Composer Studio v5.5
//******************************************************************************
#include <msp430fr4133.h>

void Init_GPIO(void);
int main(void)
{
    WDTCTL = WDTPW | WDTHOLD;                                              // Stop watchdog timer
    
    // Configure GPIO
    Init_GPIO();
    P1OUT &= ~BIT0;                                                        // Clear P1.0 output latch for a defined power-on state
    P1DIR |= BIT0;                                                         // Set P1.0 to output direction
    // Disable the GPIO power-on default high-impedance mode
    // to activate previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;

    // Configure the CS
    CSCTL5 |= DIVM_1;

    CSCTL1 |= DCOFSEL_5;                                                   // Set DCO = 16MHz, max in G6021
    CSCTL2 = FLLD_0 + 487;                                                 // DCODIV = 16MHz
    CSCTL3 |= SELREF__REFOCLK;                                             // Set REFO as FLL reference source
    __delay_cycles(7 * 31 * 16);                                           // Requires 7 reference clock delay before polling FLLUNLOCK bits
    while(CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1));                             // FLL locked
    CSCTL4 |= SELMS__DCOCLKDIV | SELA__REFOCLK;                               // set default REFO(~32768Hz) as ACLK source, ACLK = 32768Hz
                                                                           // default DCODIV as MCLK and SMCLK source
    CSCTL5 &= DIVM_1;                                                      // SMCLK = MCLK = DCODIV = 16MHz

    // Configure RTC
    RTCMOD = 10;                                                           // Interrupt and reset happen every 10*1000*(1/10KHz) = ~1S
    RTCCTL |= RTCSS__VLOCLK | RTCSR |RTCPS__1000;
    RTCCTL |= RTCIE;
   
    while(1)
    {  
        // Configure the CS
        CSCTL1 &= ~(DCOFSEL_7);                                            // Clear DCO frequency select bits first
        CSCTL1 |= DCOFSEL_2;                                               // Set DCO = 4MHz
        CSCTL2 = FLLD_0 + 121;                                             // MCLK down to 4MHz
        __delay_cycles(7 * 31 * 4);                                           // Requires 7 reference clock delay before polling FLLUNLOCK bits
        while(CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1));                             // FLL locked

        CSCTL1 &= ~(DCOFSEL_7);                                            // Clear DCO frequency select bits first
        CSCTL1 |= DCOFSEL_0;                                               // Set DCO = 1MHz
        CSCTL2 = FLLD_0 + 30;                                              // MCLK down to 1MHz
        __delay_cycles(7 * 31);                                           // Requires 7 reference clock delay before polling FLLUNLOCK bits
        while(CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1));                             // FLL locked

        P1OUT &= ~BIT0; 
        __bis_SR_register(LPM3_bits );                                // Enter LPM3, Stop all clocks
    }
}

// RTC interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=RTC_VECTOR
__interrupt void RTC_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(RTC_VECTOR))) RTC_ISR (void)
#else
#error Compiler not supported!
#endif
{
    // Configure the CS
    __bic_SR_register(SCG0);                                              // enable FLL

    CSCTL1 &= ~(DCOFSEL_7);                                               // Clear DCO frequency select bits first
    CSCTL1 |= DCOFSEL_2;                                                  // Set DCO = 4MHz
    CSCTL2 = FLLD_0 + 121;                                                // MCLK up to 4MHz
    __delay_cycles(7 * 31 * 4);                                           // Requires 7 reference clock delay before polling FLLUNLOCK bits
    while(CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1));                            // FLL locked

    CSCTL5 |= DIVM_1;
    CSCTL1 &= ~(DCOFSEL_7);                                               // Clear DCO frequency select bits first
    CSCTL1 |= DCOFSEL_5;                                                  // Set DCO = 16MHz
    CSCTL2 = FLLD_0 + 487;                                                // MCLK up to 16MHz
    __delay_cycles(7 * 31 * 16);                                          // Requires 7 reference clock delay before polling FLLUNLOCK bits
    while(CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1));                            // FLL locked
    CSCTL5 &= DIVM_1;                                                     // SMCLK = MCLK = DCODIV = 16MHz

    switch(__even_in_range(RTCIV,RTCIF))
    {
        case RTCIV_NONE :  break;                                         // No interrupt
        case RTCIV_RTCIF:  __bic_SR_register_on_exit(LPM3_bits);          // RTC Overflow
                                                                          // Exit LPM3 on reti
                           break;
        default: break;
    }
}

void Init_GPIO()
{
    P1DIR = 0xFF; P2DIR = 0xFF; P3DIR = 0xFF; P4DIR = 0xFF; 
    P5DIR = 0xFF; P6DIR = 0xFF; P7DIR = 0xFF; P8DIR = 0xFF; 
    P1REN = 0xFF; P2REN = 0xFF; P3REN = 0xFF; P4REN = 0xFF;
    P5REN = 0xFF; P6REN = 0xFF; P7REN = 0xFF; P8REN = 0xFF;
    P1OUT = 0x00; P2OUT = 0x00; P3OUT = 0x00; P4OUT = 0x00; 
    P5OUT = 0x00; P6OUT = 0x00; P7OUT = 0x00; P8OUT = 0x00;
}
