/*
 * SysDataDefinition.c
 *
 *  Created on: Sep 26, 2013
 *      Author: a0282243
 */
#include "SysDataDefinition.h"

const unsigned char ModeArray[] = { ModeAuto, cold, dehumidity, wind, heat };
const unsigned char FanSpeedArray[] = { FanSpeedAuto, ultra_low, low, middle,
		large, strong };
const unsigned char FanLeftRightFanMode[] = { position_none, position_5 };
const unsigned char FanUpdwnFanMode[] = { position_none, position_5 };
const unsigned char TemperatureMode[] = { Cels, Fah };
const unsigned char TemperaturePositionMode[] = { Temp_none, Temp_inside,
		Temp_outside };
const unsigned char TimerMode1[] = { Timer_none1, Timer_startup };
const unsigned char TimerMode2[] = { Timer_none2, Timer_shutdown };
const unsigned char SleepMode[] = { sleepmode_start, sleepmode_1, sleepmode_2,
		sleepmode_end };
const unsigned char AuxHeatmode[] = { aux_none, dry, aux_heat };
const unsigned char KeyValueArray[] = { Key_On_off, Key_Mode, Key_Fan_Speed,
		Key_Plus, Key_Minus, Key_Sleep, Key_TimerDelay, Key_SavePower, Key_Dry,
		Key_Health, Key_Light, Key_Mute, Key_Max_Power, Key_UpDwn_wind,
		Key_LeftRt_Wind, Key_Lock };
const unsigned char BehaviorPhase[] = { bringup, idle, keyscan, keyprocess,
		infrasend };
const unsigned char BatteryValue[] = { none, stage1, stage2, stage3 };
const unsigned char CelsTemperatureRange[CELS_TEMP_NUM] = { 16, 17, 18, 19, 20,
		21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32 };
const unsigned char FahTemperatureRange[FATH_TEMP_NUM] = { 60, 61, 62, 63, 64,
		65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80 };
const unsigned char TimerValue[Timer_Value_NUM] = { 0, 1, 2, 3, 4, 5, 6, 7, 8,
		9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23 };
const unsigned char TimerMinuterValue[Timer_Minuter_Value_NUM] = { 0, 1, 2, 3,
		4, 5 };
SysDataStruct SysData;

CircleLink SleepModeCircleLink[sizeof(SleepMode)];
CircleLink ModeCircleLink[sizeof(ModeArray)];
CircleLink FanSpeedCircleLink[sizeof(FanSpeedArray)];
CircleLink FanLeftRigthModeCircleLink[sizeof(FanLeftRightFanMode)];
CircleLink FanUpDwnModeCircleLink[sizeof(FanUpdwnFanMode)];
CircleLink CelsTempCircleLink[sizeof(CelsTemperatureRange)];
CircleLink FahTempCircleLink[sizeof(FahTemperatureRange)];
CircleLink TemperaturePositionCircleLink[sizeof(TemperaturePositionMode)];
CircleLink TimerMode1CircleLink[sizeof(TimerMode1)];
CircleLink TimerMode2CircleLink[sizeof(TimerMode2)];
CircleLink TempModeCircleLink[sizeof(TemperatureMode)];
CircleLink TimerValueCircleLink[sizeof(TimerValue)];
CircleLink TimerMinuterValueCircleLink[sizeof(TimerMinuterValue)];
CircleLink AuxHeatCircleLink[sizeof(AuxHeatmode)];

void CreatCycleLink(unsigned char DirectionType, CircleLink *LinkSource,
		const unsigned char *array);
void CreatCycleLink2(unsigned char DirectionType, CircleLink *LinkSource,
		const unsigned char *array);

void InitSysData(void) {
	unsigned char i = 0;

	//Initialize data cycle links
	CreatCycleLink(SINGLE_DIRECTION, SleepModeCircleLink, SleepMode);
	CreatCycleLink(SINGLE_DIRECTION, ModeCircleLink, ModeArray);
	CreatCycleLink(SINGLE_DIRECTION, FanSpeedCircleLink, FanSpeedArray);
	CreatCycleLink(SINGLE_DIRECTION, FanLeftRigthModeCircleLink,
			FanLeftRightFanMode);
	CreatCycleLink(SINGLE_DIRECTION, FanUpDwnModeCircleLink, FanUpdwnFanMode);
	CreatCycleLink(SINGLE_DIRECTION, TemperaturePositionCircleLink,
			TemperaturePositionMode);
	CreatCycleLink(SINGLE_DIRECTION, TimerMode1CircleLink, TimerMode1);
	CreatCycleLink(SINGLE_DIRECTION, TimerMode2CircleLink, TimerMode2);
	CreatCycleLink(DUAL_DIRECTION, CelsTempCircleLink, CelsTemperatureRange);
	CreatCycleLink(DUAL_DIRECTION, FahTempCircleLink, FahTemperatureRange);
	CreatCycleLink(SINGLE_DIRECTION, TempModeCircleLink, TemperatureMode);
	CreatCycleLink2(DUAL_DIRECTION, TimerValueCircleLink, TimerValue);
	CreatCycleLink2(DUAL_DIRECTION, TimerMinuterValueCircleLink,
			TimerMinuterValue);
	CreatCycleLink(SINGLE_DIRECTION, AuxHeatCircleLink, AuxHeatmode);

	for (i = 0; i < MODE_NUM; i++) {
		SysData.SysMode[i].Auto = ON;
		SysData.SysMode[i].Dry = OFF;
		SysData.SysMode[i].FanSpeed = FanSpeedCircleLink;
		SysData.SysMode[i].Healthy = OFF;
		SysData.SysMode[i].Heat = OFF;
		SysData.SysMode[i].LeftRightFanMode = FanLeftRigthModeCircleLink;
		SysData.SysMode[i].MaxPower = OFF;
		SysData.SysMode[i].SleepMode = SleepModeCircleLink;
		SysData.SysMode[i].Temperature = CelsTempCircleLink;
		SysData.SysMode[i].UpDwnFanMode = FanUpDwnModeCircleLink;
		SysData.SysMode[i].ModeName = &ModeCircleLink[i];
		SysData.SysMode[i].AuxHeat = AuxHeatCircleLink;
	}

	SysData.SysMode[4].AuxHeat = &AuxHeatCircleLink[2];

	SysData.CurrentMode = &SysData.SysMode[0];
	SysData.SysStatus.BehavirPhase = bringup;
	SysData.SysStatus.Mute = OFF;
	SysData.SysStatus.SysLock = OFF;
	SysData.SysStatus.SysON = ON;
	SysData.SysStatus.TemperatureMode = Cels;
	SysData.SysStatus.TimerDelayMinutesLeft = 0;
	SysData.SysStatus.TimerDelayON = OFF;
	SysData.SysStatus.keypressed = 0;
	SysData.SysStatus.RTCtimerInterruptFlag = 0;
	SysData.SysStatus.KeydelayInterruptFlag = 0;
	SysData.SysStatus.KeyShift = 0;
	SysData.SysStatus.batterylevel = stage3;
	SysData.SysStatus.TempPositon = TemperaturePositionCircleLink;
	SysData.SysStatus.TimerMode1 = TimerMode1CircleLink;
	SysData.SysStatus.TimerMode2 = TimerMode2CircleLink;
	SysData.SysStatus.TimerValue = TimerValueCircleLink;
	SysData.SysStatus.TimerMinuterValue = TimerMinuterValueCircleLink;

}

void CreatCycleLink(unsigned char DirectionType, CircleLink *LinkSource,
		const unsigned char *array) {
	unsigned char i = 1;
	unsigned char array_length = 1;
	while (array[i] != 0) {
		array_length++;
		i++;
	}

	if (DirectionType == SINGLE_DIRECTION) {
		for (i = 0; i < array_length; i++) {
			(*(LinkSource + i)).element = *(array + i);
			(*(LinkSource + i)).previous = LinkSource + i;
			if (i == (array_length - 1))
				(*(LinkSource + i)).next = LinkSource;
			else
				(*(LinkSource + i)).next = LinkSource + i + 1;
		}
	}

	if (DirectionType == DUAL_DIRECTION) {
		for (i = 0; i < array_length; i++) {
			(*(LinkSource + i)).element = *(array + i);
			if (i == 0)
				(*(LinkSource + i)).previous = LinkSource + i;
			else
				(*(LinkSource + i)).previous = LinkSource + i - 1;

			if (i == (array_length - 1))
				(*(LinkSource + i)).next = LinkSource + i;
			else
				(*(LinkSource + i)).next = LinkSource + i + 1;
		}
	}
}

void CreatCycleLink2(unsigned char DirectionType, CircleLink *LinkSource,
		const unsigned char *array) {
	unsigned char i = 1;
	unsigned char array_length = 1;
	while (array[i] != 0) {
		array_length++;
		i++;
	}

	if (DirectionType == SINGLE_DIRECTION) {
		for (i = 0; i < array_length; i++) {
			(*(LinkSource + i)).element = *(array + i);
			(*(LinkSource + i)).previous = LinkSource + i;
			if (i == (array_length - 1))
				(*(LinkSource + i)).next = LinkSource;
			else
				(*(LinkSource + i)).next = LinkSource + i + 1;
		}
	}

	if (DirectionType == DUAL_DIRECTION) {
		for (i = 0; i < array_length; i++) {
			(*(LinkSource + i)).element = *(array + i);
			if (i == 0)
				(*(LinkSource + i)).previous = LinkSource + (array_length - 1);
			else
				(*(LinkSource + i)).previous = LinkSource + i - 1;

			if (i == (array_length - 1))
				(*(LinkSource + i)).next = LinkSource;
			else
				(*(LinkSource + i)).next = LinkSource + i + 1;
		}
	}
}
